# -*- coding: utf-8 -*-

"""
Framework agnostic PEM file parsing functions.
"""

from __future__ import absolute_import, division, print_function

import hashlib
import re

from ._compat import PY3, unicode


class _Base(object):
    """
    Base class for parsed objects.
    """
    def __init__(self, _pem_bytes):
        if isinstance(_pem_bytes, unicode):
            _pem_bytes = _pem_bytes.encode('ascii')
        self._pem_bytes = _pem_bytes

    if PY3:
        def __str__(self):
            return self._pem_bytes.decode('ascii')
    else:
        def __str__(self):
            return self._pem_bytes

    def __repr__(self):
        return '<{0}(PEM string with SHA-1 digest {1!r})>'.format(
            self.__class__.__name__,
            hashlib.sha1(self._pem_bytes).hexdigest()
        )

    def as_bytes(self):
        """
        Return the PEM-encoded content as :obj:`bytes`.
        """
        return self._pem_bytes

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (type(self) == type(other) and
                self._pem_bytes == other._pem_bytes)

    def __ne__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (type(self) != type(other) or
                self._pem_bytes != other._pem_bytes)

    def __hash__(self):
        return hash(self._pem_bytes)


class Certificate(_Base):
    """
    A certificate.
    """


class CertificateRequest(_Base):
    """
    A certificate signing request.
    """


class Key(_Base):
    """
    A secret key of unknown type.
    """


class RSAPrivateKey(Key):
    """
    A secret RSA key.
    """


class DHParameters(_Base):
    """
    Diffie-Hellman parameters for DHE.
    """


_PEM_TO_CLASS = {
    b"CERTIFICATE": Certificate,
    b"PRIVATE KEY": Key,
    b"RSA PRIVATE KEY": RSAPrivateKey,
    b"DH PARAMETERS": DHParameters,
    b"NEW CERTIFICATE REQUEST": CertificateRequest,
    b"CERTIFICATE REQUEST": CertificateRequest,
}
_PEM_RE = re.compile(
    b"-----BEGIN (" + b"|".join(_PEM_TO_CLASS.keys()) + b""")-----\r?
.+?\r?
-----END \\1-----\r?\n?""", re.DOTALL)


def parse(pem_str):
    """
    Extract PEM objects from *pem_str*.

    :param pem_str: String to parse.
    :type pem_str: bytes
    :return: list of :ref:`pem-objects`
    """
    return [_PEM_TO_CLASS[match.group(1)](match.group(0))
            for match in _PEM_RE.finditer(pem_str)]


def parse_file(file_name):
    """
    Read *file_name* and parse PEM objects from it using :func:`parse`.
    """
    with open(file_name, 'rb') as f:
        return parse(f.read())



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import certifi

import pem

from .data import (
    CERT_NO_NEW_LINE,
    CERT_PEMS,
    CERT_PEMS_NO_NEW_LINE,
    DH_PEM,
    KEY_PEM,
)


# SHA-1 of "test"
TEST_DIGEST = (
    "PEM string with SHA-1 digest "
    "'a94a8fe5ccb19ba61c4c0873d391e987982fbbd3'"
)


class TestPEMObjects(object):
    def test_cert_has_correct_repr(self):
        """
        Calling repr on a Certificate instance returns the proper string.
        """
        cert = pem.Certificate(b"test")

        assert "<Certificate({0})>".format(TEST_DIGEST) == repr(cert)

    def test_cert_has_correct_str(self):
        """
        Calling str on a Certificate instance returns the proper string.
        """
        cert = pem.Certificate(b"test")

        assert str(cert) == "test"

    def test_cert_req_has_correct_repr(self):
        """
        Calling repr on a CertificateRequest instance returns the proper
        string.
        """
        cert_req = pem.CertificateRequest(b"test")

        assert "<CertificateRequest({0})>".format(
            TEST_DIGEST
        ) == repr(cert_req)

    def test_cert_req_has_correct_str(self):
        """
        Calling str on a CertificateRequest instance returns the proper string.
        """
        cert_req = pem.CertificateRequest(b"test")

        assert str(cert_req) == "test"

    def test_key_has_correct_repr(self):
        """
        Calling repr on a Key instance returns the proper string.
        """
        key = pem.Key(b"test")

        assert "<Key({0})>".format(TEST_DIGEST) == repr(key)

    def test_key_has_correct_str(self):
        """
        Calling str on a Key instance returns the proper string.
        """
        key = pem.Key(b"test")

        assert str(key) == "test"

    def test_rsa_key_has_correct_repr(self):
        """
        Calling repr on a RSAPrivateKey instance returns the proper string.
        """
        key = pem.RSAPrivateKey(b"test")

        assert "<RSAPrivateKey({0})>".format(TEST_DIGEST) == repr(key)

    def test_rsa_key_has_correct_str(self):
        """
        Calling str on a RSAPrivateKey instance returns the proper string.
        """
        key = pem.RSAPrivateKey(b"test")

        assert str(key) == "test"

    def test_dh_params_has_correct_repr(self):
        """
        Calling repr on a DHParameters instance returns the proper string.
        """
        params = pem.DHParameters(b"test")

        assert "<DHParameters({0})>".format(TEST_DIGEST) == repr(params)

    def test_dh_params_has_correct_str(self):
        """
        Calling str on a DHParameters instance returns the proper string.
        """
        params = pem.DHParameters(b"test")

        assert str(params) == "test"

    def test_certificate_unicode(self):
        """
        Passing unicode to Certificate encodes the string as ASCII.
        """
        cert = pem.Certificate(u'a string')

        assert cert.as_bytes() == b'a string'
        assert str(cert) == 'a string'

    def test_certificate_request_unicode(self):
        """
        Passing unicode to CertificateRequest encodes the string as ASCII.
        """
        cert_req = pem.CertificateRequest(u'a string')

        assert cert_req.as_bytes() == b'a string'
        assert str(cert_req) == 'a string'

    def test_key_unicode(self):
        """
        Passing unicode to Key encodes the string as ASCII.
        """
        key = pem.Key(u'a string')

        assert key.as_bytes() == b'a string'
        assert str(key) == 'a string'

    def test_rsa_key_unicode(self):
        """
        Passing unicode to RSAPrivateKey encodes the string as ASCII.
        """
        key = pem.RSAPrivateKey(u'a string')

        assert key.as_bytes() == b'a string'
        assert str(key) == 'a string'

    def test_dhparams_unicode_deprecated(self):
        """
        Passing unicode to DHParameters encodes the string as ASCII.
        """
        params = pem.DHParameters(u'a string')

        assert params.as_bytes() == b'a string'
        assert str(params) == 'a string'

    def test_certs_equal(self):
        """
        Two Certificate instances with equal contents are equal.
        """
        cert1 = pem.Certificate(b"test")
        cert2 = pem.Certificate(b"test")

        assert cert1 == cert2
        assert cert2 == cert1
        assert hash(cert1) == hash(cert2)

    def test_cert_reqs_equal(self):
        """
        Two Certificate Request instances with equal contents are equal.
        """
        cert_req1 = pem.CertificateRequest(b"test")
        cert_req2 = pem.CertificateRequest(b"test")

        assert cert_req1 == cert_req2
        assert cert_req2 == cert_req1
        assert hash(cert_req1) == hash(cert_req2)

    def test_keys_equal(self):
        """
        Two Key instances with equal contents are equal and have equal hashes.
        """
        key1 = pem.Key(b"test")
        key2 = pem.Key(b"test")

        assert key1 == key2
        assert key2 == key1
        assert hash(key1) == hash(key2)

    def test_rsa_keys_equal(self):
        """
        Two RSAPrivateKey instances with equal contents are equal and have
        equal hashes.
        """

        key1 = pem.RSAPrivateKey(b"test")
        key2 = pem.RSAPrivateKey(b"test")
        assert key1 == key2
        assert key2 == key1
        assert hash(key1) == hash(key2)

    def test_dh_params_equal(self):
        """
        Two DHParameters instances with equal contents are equal and have equal
        hashes.
        """
        params1 = pem.DHParameters(b"test")
        params2 = pem.DHParameters(b"test")

        assert params1 == params2
        assert params2 == params1
        assert hash(params1) == hash(params2)

    def test_cert_contents_unequal(self):
        """
        Two Certificate instances with unequal contents are not equal.
        """
        cert1 = pem.Certificate(b"test1")
        cert2 = pem.Certificate(b"test2")

        assert cert1 != cert2
        assert cert2 != cert1

    def test_cert_req_contents_unequal(self):
        """
        Two CertificateRequest instances with unequal contents are not equal.
        """
        cert_req1 = pem.CertificateRequest(b"test1")
        cert_req2 = pem.CertificateRequest(b"test2")
        assert cert_req1 != cert_req2
        assert cert_req2 != cert_req1

    def test_different_objects_unequal(self):
        """
        Two PEM objects of different types but with equal contents are not
        equal.
        """
        cert = pem.Certificate(b"test")
        cert_req = pem.CertificateRequest(b"test")
        key = pem.Key(b"test")
        rsa_key = pem.RSAPrivateKey(b"test")

        assert not cert == key
        assert cert != key
        assert key != cert
        assert cert != cert_req
        assert cert_req != cert
        assert key != rsa_key
        assert rsa_key != key

    def test_incompatible_types(self):
        """
        A PEM object is not equal to some other arbitrary object.
        """
        cert = pem.Certificate(b"test")

        assert not cert == object()
        assert cert != object()
        assert object() != cert


class TestParse(object):
    def test_key(self):
        """
        Parses a PEM string with a key into an RSAPrivateKey.
        """
        rv = pem.parse(KEY_PEM)
        key, = rv
        assert isinstance(key, pem.RSAPrivateKey)
        assert KEY_PEM == key.as_bytes()

    def test_certificates(self):
        """
        Parses a PEM string with multiple certificates into a list of
        corresponding Certificates.
        """
        certs = pem.parse(b''.join(CERT_PEMS))
        assert all(isinstance(c, pem.Certificate) for c in certs)
        assert CERT_PEMS == [cert.as_bytes() for cert in certs]

    def test_certificate_no_new_line(self):
        """
        Parses a PEM string without a new line at the end
        """
        cert, = pem.parse(CERT_NO_NEW_LINE)
        assert isinstance(cert, pem.Certificate)
        assert CERT_NO_NEW_LINE == cert.as_bytes()

    def test_certificates_no_new_line(self):
        """
        Parses a PEM string with multiple certificates without a new line
        at the end into a list of corresponding Certificates.
        """
        certs = pem.parse(b''.join(CERT_PEMS_NO_NEW_LINE))
        assert all(isinstance(c, pem.Certificate) for c in certs)
        assert CERT_PEMS_NO_NEW_LINE == [cert.as_bytes() for cert in certs]

    def test_dh(self):
        """
        Parses a PEM string with with DH parameters into a DHParameters.
        """
        rv = pem.parse(DH_PEM)
        dh, = rv
        assert isinstance(dh, pem.DHParameters)
        assert DH_PEM == dh.as_bytes()

    def test_file(self, tmpdir):
        """
        A file with multiple certificate PEMs is parsed into a list of
        corresponding Certificates.
        """
        certs_file = tmpdir.join('certs.pem')
        certs_file.write(b''.join(CERT_PEMS))
        certs = pem.parse_file(str(certs_file))
        assert all(isinstance(c, pem.Certificate) for c in certs)
        assert CERT_PEMS == [cert.as_bytes() for cert in certs]

    def test_loads_certifi(self):
        """
        Loading certifi returns a list of Certificates.
        """
        cas = pem.parse_file(certifi.where())
        assert isinstance(cas, list)
        assert all(isinstance(ca, pem.Certificate) for ca in cas)

    def test_allows_lf(self):
        """
        \n and \r\n are treated equal.
        """
        lf_pem = KEY_PEM.replace(b"\n", b"\r\n")
        rv, = pem.parse(lf_pem)
        assert rv.as_bytes() == lf_pem



# -*- coding: utf-8 -*-

"""
Twisted-specific convenience helpers.
"""

from __future__ import absolute_import, division, print_function

import warnings

from OpenSSL.SSL import FILETYPE_PEM
from twisted.internet import ssl

from ._core import parse_file, Certificate, DHParameters, Key


def certificateOptionsFromPEMs(pemObjects, **kw):
    """
    Load a CertificateOptions from the given collection of PEM objects
    (already-loaded private keys and certificates).

    In those PEM objects, identify one private key and its corresponding
    certificate to use as the primary certificate.  Then use the rest of the
    certificates found as chain certificates.  Raise a ValueError if no
    certificate matching a private key is found.

    :return: A TLS context factory using *pemObjects*
    :rtype: `twisted.internet.ssl.CertificateOptions`_

    .. _`twisted.internet.ssl.CertificateOptions`: \
        https://twistedmatrix.com/documents/current/api/\
        twisted.internet.ssl.CertificateOptions.html
    """
    keys = [key for key in pemObjects if isinstance(key, Key)]
    if not len(keys):
        raise ValueError('Supplied PEM file(s) does *not* contain a key.')
    if len(keys) > 1:
        raise ValueError('Supplied PEM file(s) contains *more* than one key.')

    privateKey = ssl.KeyPair.load(str(keys[0]), FILETYPE_PEM)

    certs = [cert for cert in pemObjects if isinstance(cert, Certificate)]
    if not len(certs):
        raise ValueError('*At least one* certificate is required.')
    certificates = [ssl.Certificate.loadPEM(str(certPEM))
                    for certPEM in certs]

    certificatesByFingerprint = dict(
        [(certificate.getPublicKey().keyHash(), certificate)
         for certificate in certificates]
    )

    if privateKey.keyHash() not in certificatesByFingerprint:
        raise ValueError("No certificate matching {fingerprint} found.".format(
            fingerprint=privateKey.keyHash()
        ))

    primaryCertificate = certificatesByFingerprint.pop(privateKey.keyHash())

    if "dhParameters" in kw:
        warnings.warn(
            "Passing DH parameters as a keyword argument instead of a PEM "
            "object is deprecated as of pem 16.1.0.",
            DeprecationWarning
        )
    else:
        dhparams = [o for o in pemObjects if isinstance(o, DHParameters)]
        if len(dhparams) > 1:
            raise ValueError(
                "Supplied PEM file(s) contain(s) *more* than one set of DH "
                "parameters.")
        elif len(dhparams) == 1:
            kw["dhParameters"] = DiffieHellmanParameters(str(dhparams[0]))

    fakeEDHSupport = "dhParameters" in kw and not _DH_PARAMETERS_SUPPORTED
    if fakeEDHSupport:
        dhParameters = kw.pop("dhParameters")

    if _DH_PARAMETERS_SUPPORTED is False:
        warnings.warn(
            "Using pem with Twisted older than 14.0.0 is deprecated as of pem"
            " 15.0.0.  "
            "The backport of DiffieHellmanParameters will be removed.",
            DeprecationWarning
        )

    ctxFactory = ssl.CertificateOptions(
        privateKey=privateKey.original,
        certificate=primaryCertificate.original,
        extraCertChain=[chain.original
                        for chain in certificatesByFingerprint.values()],
        **kw
    )

    if fakeEDHSupport:
        return _DHParamContextFactory(ctxFactory, dhParameters)
    else:
        return ctxFactory


def certificateOptionsFromFiles(*pemFiles, **kw):
    """
    Read all files named by *pemFiles*, and parse them using
    :func:`certificateOptionsFromPEMs`.
    """
    pems = []
    for pemFile in pemFiles:
        pems += parse_file(pemFile)
    return certificateOptionsFromPEMs(pems, **kw)


class _DHParamContextFactory(object):
    """
    A wrapping context factory that gets a context from a different
    context factory and then sets temporary DH params on it. This
    enables PFS ciphersuites using DHE.
    """
    def __init__(self, ctxFactory, dhParameters):
        self.ctxFactory = ctxFactory
        self.dhParameters = dhParameters

    def getContext(self):
        ctx = self.ctxFactory.getContext()
        ctx.load_tmp_dh(self.dhParameters._dhFile.path)
        return ctx


class _DiffieHellmanParameters(object):
    """
    A representation of key generation parameters that are required for
    Diffie-Hellman key exchange.
    """
    def __init__(self, parameters):
        self._dhFile = parameters

    @classmethod
    def fromFile(cls, filePath):
        """
        Load parameters from a file.

        Such a file can be generated using the C{openssl} command line tool as
        following:

        C{openssl dhparam -out dh_param_1024.pem -2 1024}

        Please refer to U{OpenSSL's C{dhparam} documentation
        <http://www.openssl.org/docs/apps/dhparam.html>} for further details.

        @param filePath: A file containing parameters for Diffie-Hellman key
            exchange.
        @type filePath: L{FilePath <twisted.python.filepath.FilePath>}

        @return: A instance that loads its parameters from C{filePath}.
        @rtype: L{DiffieHellmanParameters
            <twisted.internet.ssl.DiffieHellmanParameters>}
        """
        return cls(filePath)


try:
    from twisted.internet.ssl import DiffieHellmanParameters
    _DH_PARAMETERS_SUPPORTED = True
except ImportError:  # pragma: nocover
    DiffieHellmanParameters = _DiffieHellmanParameters
    _DH_PARAMETERS_SUPPORTED = False



from __future__ import absolute_import, division, print_function

try:
    from . import twisted
except ImportError:
    twisted = None
from ._core import (
    Certificate,
    CertificateRequest,
    DHParameters,
    Key,
    RSAPrivateKey,
    parse,
    parse_file,
)


__version__ = "17.1.0"
__author__ = "Hynek Schlawack"
__license__ = "MIT"
__description__ = "Easy PEM file parsing in Python."
__uri__ = "https://pem.readthedocs.io/"
__email__ = "hs@ox.cx"


_DEPRECATION_WARNING = (
    "Calling {func} from the pem package is deprecated as of pem 15.0.0.  "
    "Please use pem.twisted.{func} instead."
)


def certificateOptionsFromFiles(*args, **kw):
    """
    Deprecated function.  Please use pem.twisted.certificateOptionsFromFiles.
    """
    import warnings

    from .twisted import certificateOptionsFromFiles

    warnings.warn(
        _DEPRECATION_WARNING.format(func="certificateOptionsFromFiles"),
        DeprecationWarning
    )
    return certificateOptionsFromFiles(*args, **kw)


def certificateOptionsFromPEMs(*args, **kw):
    """
    Deprecated function.  Please use pem.twisted.certificateOptionsFromPEMs.
    """
    import warnings

    from .twisted import certificateOptionsFromPEMs

    warnings.warn(
        _DEPRECATION_WARNING.format(func="certificateOptionsFromPEMs"),
        DeprecationWarning
    )
    return certificateOptionsFromPEMs(*args, **kw)


__all__ = [
    "Certificate",
    "CertificateRequest",
    "DHParameters",
    "Key",
    "RSAPrivateKey",
    "certificateOptionsFromFiles",
    "certificateOptionsFromPEMs",
    "parse",
    "parse_file",
    "twisted",
]



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


import pytest

from OpenSSL import crypto, SSL
from pretend import call, call_recorder, stub

ssl = pytest.importorskip("twisted.internet.ssl")

import pem

from pem.twisted import certificateOptionsFromFiles

from .data import KEY_PEM, KEY_PEM2, CERT_PEMS, DH_PEM


@pytest.fixture
def keyCertChainDHFile(tmpdir):
    """
    Returns a file containing the key, three certificates, and DH parameters.
    """
    pemFile = tmpdir.join('key_cert_and_chain_and_params.pem')
    pemFile.write(KEY_PEM + b''.join(CERT_PEMS) + DH_PEM)

    return pemFile


@pytest.fixture
def keyCertChainFile(tmpdir):
    """
    Returns a file containing the key and three certificates.
    """
    pemFile = tmpdir.join('key_cert_and_chain.pem')
    pemFile.write(KEY_PEM + b''.join(CERT_PEMS))

    return pemFile


class TestCertificateOptionsFromFiles(object):
    def test_worksWithoutChain(self, tmpdir):
        """
        Creating CO without chain certificates works.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM)
        certFile = tmpdir.join('cert.pem')
        certFile.write(CERT_PEMS[0])

        ctxFactory = certificateOptionsFromFiles(
            str(keyFile), str(certFile),
        )

        assert [] == ctxFactory.extraCertChain

    def test_worksWithChainInExtraFile(self, tmpdir):
        """
        Chain can be in a separate file.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM)
        certFile = tmpdir.join('cert.pem')
        certFile.write(CERT_PEMS[0])
        chainFile = tmpdir.join('chain.pem')
        chainFile.write(b''.join(CERT_PEMS[1:]))

        ctxFactory = certificateOptionsFromFiles(
            str(keyFile), str(certFile), str(chainFile)
        )

        assert 2 == len(ctxFactory.extraCertChain)

    def test_worksWithChainInSameFile(self, tmpdir):
        """
        Chain can be in the same file as the certificate.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM)
        certFile = tmpdir.join('cert_and_chain.pem')
        certFile.write(b''.join(CERT_PEMS))

        ctxFactory = certificateOptionsFromFiles(
            str(keyFile), str(certFile)
        )

        assert 2 == len(ctxFactory.extraCertChain)

    def test_useTypesNotOrdering(self, tmpdir):
        """
        L{pem.certificateOptionsFromFiles} identifies the chain, key, and
        certificate for Twisted's L{CertificateOptions} based on their types
        and certificate fingerprints, not their order within the file.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM)
        certFile = tmpdir.join('cert_and_chain.pem')
        certFile.write(b''.join(reversed(CERT_PEMS)))

        ctxFactory = certificateOptionsFromFiles(
            str(keyFile), str(certFile)
        )

        assert 2 == len(ctxFactory.extraCertChain)

    def test_worksWithEverythingInOneFile(self, keyCertChainDHFile):
        """
        Key, certificate, and chain can also be in a single file.
        """
        ctxFactory = certificateOptionsFromFiles(str(keyCertChainDHFile))

        assert 2 == len(ctxFactory.extraCertChain)
        assert ctxFactory.dhParameters is not None

    def test_passesCertsInCorrectFormat(self, keyCertChainDHFile):
        """
        PEM objects are correctly detected and passed into CO.
        """
        ctxFactory = certificateOptionsFromFiles(str(keyCertChainDHFile))

        assert isinstance(ctxFactory.privateKey, crypto.PKey)
        assert isinstance(ctxFactory.certificate, crypto.X509)
        assert all(isinstance(cert, crypto.X509)
                   for cert in ctxFactory.extraCertChain)

    def test_forwardsKWargs(self, keyCertChainDHFile):
        """
        Extra keyword arguments are passed into CO.
        """
        ctxFactory = certificateOptionsFromFiles(
            str(keyCertChainDHFile),
            method=SSL.TLSv1_METHOD,
        )

        assert SSL.TLSv1_METHOD is ctxFactory.method

    def test_catchesMissingKey(self, tmpdir):
        """
        Raises ValueError if a key is missing.
        """
        certFile = tmpdir.join('cert_and_chain.pem')
        certFile.write(b''.join(CERT_PEMS))

        with pytest.raises(ValueError):
            certificateOptionsFromFiles(
                str(certFile)
            )

    def test_catchesMultipleKeys(self, tmpdir):
        """
        Raises ValueError if multiple keys are present.
        """
        allFile = tmpdir.join('key_cert_and_chain.pem')
        allFile.write(KEY_PEM + b''.join(CERT_PEMS) + KEY_PEM2)

        with pytest.raises(ValueError):
            certificateOptionsFromFiles(
                str(allFile)
            )

    def test_catchesMissingCertificate(self, tmpdir):
        """
        Raises ValueError if no certificate is passed.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM)

        with pytest.raises(ValueError):
            certificateOptionsFromFiles(
                str(keyFile)
            )

    def test_catchesKeyCertificateMismatch(self, tmpdir):
        """
        A ValueError is raised when some certificates are present in the pem,
        but no certificate in the pem matches the key.
        """
        keyFile = tmpdir.join('key.pem')
        keyFile.write(KEY_PEM + b"".join(CERT_PEMS[1:]))

        with pytest.raises(ValueError) as excinfo:
            certificateOptionsFromFiles(
                str(keyFile)
            )

        assert str(excinfo.value).startswith("No certificate matching ")

    def test_catchesMultipleDHParams(self, tmpdir):
        """
        A ValueError is raised when more than one set of DH parameters is
        present.
        """
        pemFile = tmpdir.join('multiple_params.pem')
        pemFile.write(KEY_PEM + CERT_PEMS[0] + DH_PEM + DH_PEM)

        with pytest.raises(ValueError) as excinfo:
            certificateOptionsFromFiles(
                str(pemFile)
            )

        assert (
            "Supplied PEM file(s) contain(s) *more* than one set of DH "
            "parameters."
        ) == str(excinfo.value)


class TestForwardCompatibleDHE(object):
    def test_fakeDHParameterSupport(self, monkeypatch, keyCertChainFile,
                                    recwarn):
        """
        Fake DH parameter support if Twisted doesn't support it for explicitly
        passed DH parameters.

        Warns about deprecation.
        """
        fakeCtxFactory = object()
        recorder = call_recorder(lambda *a, **kw: fakeCtxFactory)
        monkeypatch.setattr(ssl, "CertificateOptions", recorder)
        monkeypatch.setattr(pem.twisted, "_DH_PARAMETERS_SUPPORTED", False)
        fakeParameters = object()

        with pytest.warns(DeprecationWarning) as ws:
            ctxFactory = certificateOptionsFromFiles(
                str(keyCertChainFile),
                dhParameters=fakeParameters
            )
            assert (
                "Passing DH parameters as a keyword argument instead of a PEM "
                "object is deprecated"
                in str(ws[0].message)
            )
            assert (
                "The backport of DiffieHellmanParameters will be removed."
                in str(ws[1].message)
            )

        assert isinstance(ctxFactory, pem.twisted._DHParamContextFactory)
        assert ctxFactory.ctxFactory is fakeCtxFactory
        assert "dhParameters" not in recorder.calls[0].kwargs

    def test_realDHParameterSupport(self, monkeypatch, keyCertChainFile,
                                    recwarn):
        """
        Pass explicitly supplied DH parameters directly to CertificateOptions
        if the installed version of Twisted supports it.

        Warns about deprecation.
        """
        fakeCtxFactory = object()
        recorder = call_recorder(lambda *a, **kw: fakeCtxFactory)
        monkeypatch.setattr(ssl, "CertificateOptions", recorder)
        monkeypatch.setattr(pem.twisted, "_DH_PARAMETERS_SUPPORTED", True)
        fakeParameters = object()

        with pytest.warns(DeprecationWarning) as ws:
            ctxFactory = certificateOptionsFromFiles(
                str(keyCertChainFile),
                dhParameters=fakeParameters
            )
            assert (
                "Passing DH parameters as a keyword argument instead of a PEM "
                "object is deprecated"
                in str(ws[0].message)
            )

        assert ctxFactory is fakeCtxFactory
        assert recorder.calls[0].kwargs["dhParameters"] == fakeParameters

    def test_fakeDHParameterFileSupport(self, monkeypatch, keyCertChainDHFile,
                                        recwarn):
        """
        Fake DH parameter support if Twisted doesn't support it for DH
        parameters loaded from file.

        Warns about deprecation.
        """
        fakeCtxFactory = object()
        recorder = call_recorder(lambda *a, **kw: fakeCtxFactory)
        monkeypatch.setattr(ssl, "CertificateOptions", recorder)
        monkeypatch.setattr(pem.twisted, "_DH_PARAMETERS_SUPPORTED", False)

        with pytest.warns(DeprecationWarning) as ws:
            ctxFactory = certificateOptionsFromFiles(
                str(keyCertChainDHFile),
            )
            assert (
                "The backport of DiffieHellmanParameters will be removed."
                in str(ws[0].message)
            )

        assert isinstance(ctxFactory, pem.twisted._DHParamContextFactory)
        assert ctxFactory.ctxFactory is fakeCtxFactory
        assert "dhParameters" not in recorder.calls[0].kwargs

    def test_realDHParameterFileSupport(self, monkeypatch, keyCertChainDHFile):
        """
        Pass DH parameters loaded from a file directly to CertificateOptions if
        the installed version of Twisted supports it.
        """
        fakeCtxFactory = object()
        recorder = call_recorder(lambda *a, **kw: fakeCtxFactory)
        monkeypatch.setattr(ssl, "CertificateOptions", recorder)
        monkeypatch.setattr(pem.twisted, "_DH_PARAMETERS_SUPPORTED", True)

        ctxFactory = certificateOptionsFromFiles(
            str(keyCertChainDHFile),
        )

        assert ctxFactory is fakeCtxFactory
        assert isinstance(
            recorder.calls[0].kwargs["dhParameters"],
            pem.twisted.DiffieHellmanParameters)

    def test_DiffieHellmanParameters(self):
        """
        Make sure lines are executed.
        """
        o = object()

        dhp = pem.twisted._DiffieHellmanParameters.fromFile(o)

        assert o is dhp._dhFile

    def test_DHParamContextFactory(self):
        """
        ContextFactory is wrapped and DH params loaded.
        """
        fakeContext = stub(
            load_tmp_dh=call_recorder(lambda dhParams: None)
        )
        fakeFactory = stub(getContext=lambda: fakeContext)
        fakeDH = stub(path=b"foo")

        ctxFactory = pem.twisted._DHParamContextFactory(
            fakeFactory, pem.twisted._DiffieHellmanParameters(fakeDH)
        )
        ctx = ctxFactory.getContext()

        assert fakeContext is ctx
        assert [call(b"foo")] == fakeContext.load_tmp_dh.calls


class TestDeprecations(object):
    def test_certificateOptionsFromFiles(self, tmpdir, recwarn):
        """
        pem.certificateOptionsFromFiles raises a deprecation warning.
        """
        keyFile = tmpdir.join("key.pem")
        keyFile.write(KEY_PEM)
        certFile = tmpdir.join("cert.pem")
        certFile.write(CERT_PEMS[0])

        with pytest.warns(DeprecationWarning) as ws:
            pem.certificateOptionsFromFiles(
                str(keyFile), str(certFile),
            )

            assert "certificateOptionsFromFiles" in str(ws[0].message)

    def test_certificateOptionsFromPEMs(self, tmpdir, recwarn):
        """
        pem.certificateOptionsFromPEMs raises a deprecation warning.
        """
        with pytest.warns(DeprecationWarning) as ws:
            pem.certificateOptionsFromPEMs(
                pem.parse(CERT_PEMS[0]) + pem.parse(KEY_PEM),
            )

            assert "certificateOptionsFromPEMs" in str(ws[0].message)
